      subroutine daxpy(n,da,dx,incx,dy,incy)
!                                                                                                                                                
!     constant times a vector plus a vector.                                                                                                     
!     uses unrolled loops for increments equal to one.                                                                                           
!     jack dongarra, linpack, 3/11/78.                                                                                                           
!                                                                                                                                                
      double precision dx(1),dy(1),da
      integer i,incx,incy,ix,iy,m,mp1,n
!                                                                                                                                                
      if(n.le.0)return
      if (da .eq. 0.0d0) return
      if(incx.eq.1.and.incy.eq.1)go to 20
!                                                                                                                                                
!        code for unequal increments or equal increments                                                                                         
!          not equal to 1                                                                                                                        
!                                                                                                                                                
      ix = 1
      iy = 1
      if(incx.lt.0)ix = (-n+1)*incx + 1
      if(incy.lt.0)iy = (-n+1)*incy + 1
      do 10 i = 1,n
        dy(iy) = dy(iy) + da*dx(ix)
        ix = ix + incx
        iy = iy + incy
   10 continue
      return
!                                                                                                                                                
!        code for both increments equal to 1                                                                                                     
!                                                                                                                                                
!                                                                                                                                                
!        clean-up loop                                                                                                                           
!                                                                                                                                                
   20 m = mod(n,4)
      if( m .eq. 0 ) go to 40
      do 30 i = 1,m
        dy(i) = dy(i) + da*dx(i)
   30 continue
      if( n .lt. 4 ) return
   40 mp1 = m + 1
      do 50 i = mp1,n,4
        dy(i) = dy(i) + da*dx(i)
        dy(i + 1) = dy(i + 1) + da*dx(i + 1)
        dy(i + 2) = dy(i + 2) + da*dx(i + 2)
        dy(i + 3) = dy(i + 3) + da*dx(i + 3)
   50 continue
      return
      end

      subroutine  dcopy(n,dx,incx,dy,incy)
!                                                                                                                                                
!     copies a vector, x, to a vector, y.                                                                                                        
!     uses unrolled loops for increments equal to one.                                                                                           
!     jack dongarra, linpack, 3/11/78.                                                                                                           
!                                                                                                                                                
      double precision dx(1),dy(1)
      integer i,incx,incy,ix,iy,m,mp1,n
!                                                                                                                                                
      if(n.le.0)return
      if(incx.eq.1.and.incy.eq.1)go to 20
!                                                                                                                                                
!        code for unequal increments or equal increments                                                                                         
!          not equal to 1                                                                                                                        
!                                                                                                                                                
      ix = 1
      iy = 1
      if(incx.lt.0)ix = (-n+1)*incx + 1
      if(incy.lt.0)iy = (-n+1)*incy + 1
      do 10 i = 1,n
        dy(iy) = dx(ix)
        ix = ix + incx
        iy = iy + incy
   10 continue
      return
!                                                                                                                                                
!        code for both increments equal to 1                                                                                                     
!                                                                                                                                                
!                                                                                                                                                
!        clean-up loop                                                                                                                           
!                                                                                                                                                
   20 m = mod(n,7)
      if( m .eq. 0 ) go to 40
      do 30 i = 1,m
        dy(i) = dx(i)
   30 continue
      if( n .lt. 7 ) return
   40 mp1 = m + 1
      do 50 i = mp1,n,7
        dy(i) = dx(i)
        dy(i + 1) = dx(i + 1)
        dy(i + 2) = dx(i + 2)
        dy(i + 3) = dx(i + 3)
        dy(i + 4) = dx(i + 4)
        dy(i + 5) = dx(i + 5)
        dy(i + 6) = dx(i + 6)
   50 continue
      return
      end

      double precision function ddot(n,dx,incx,dy,incy)
!                                                                                                                                                
!     forms the dot product of two vectors.                                                                                                      
!     uses unrolled loops for increments equal to one.                                                                                           
!     jack dongarra, linpack, 3/11/78.                                                                                                           
!                                                                                                                                                
      double precision dx(1),dy(1),dtemp
      integer i,incx,incy,ix,iy,m,mp1,n
!                                                                                                                                                
      ddot = 0.0d0
      dtemp = 0.0d0
      if(n.le.0)return
      if(incx.eq.1.and.incy.eq.1)go to 20
!                                                                                                                                                
!        code for unequal increments or equal increments                                                                                         
!          not equal to 1                                                                                                                        
!                                                                                                                                                
      ix = 1
      iy = 1
      if(incx.lt.0)ix = (-n+1)*incx + 1
      if(incy.lt.0)iy = (-n+1)*incy + 1
      do 10 i = 1,n
        dtemp = dtemp + dx(ix)*dy(iy)
        ix = ix + incx
        iy = iy + incy
   10 continue
      ddot = dtemp
      return
!                                                                                                                                                
!        code for both increments equal to 1                                                                                                     
!                                                                                                                                                
!                                                                                                                                                
!        clean-up loop                                                                                                                           
!                                                                                                                                                
   20 m = mod(n,5)
      if( m .eq. 0 ) go to 40
      do 30 i = 1,m
        dtemp = dtemp + dx(i)*dy(i)
   30 continue
      if( n .lt. 5 ) go to 60
   40 mp1 = m + 1
      do 50 i = mp1,n,5
        dtemp = dtemp + dx(i)*dy(i) + dx(i + 1)*dy(i + 1) + &
         dx(i + 2)*dy(i + 2) + dx(i + 3)*dy(i + 3) + dx(i + 4)*dy(i + 4)
   50 continue
   60 ddot = dtemp

      return
      end
